# install.tcl --
#
# Generic installer script
# (Adapted from BLT installer script)
#
# Copyright (c) 2001 Zveno Pty Ltd
# http://www.zveno.com/
#
# Zveno makes this software available free of charge for any purpose.
# Copies may be made of this software but all of this notice must be included
# on any copy.
#
# The software was developed for research purposes only and Zveno does not
# warrant that it is error free or fit for any purpose.  Zveno disclaims any
# liability for all claims, expenses, losses, damages and costs any user may
# incur as a result of using, copying or modifying this software.
#
# $Id: install.tcl,v 1.2 2001/01/17 04:58:54 balls Exp $

namespace eval Installer {
    variable datafilename install.data

    variable Package Undefined
    variable Version {Not specified}
    variable Welcome {No welcome message has been provided}

    variable commandList {}
    variable cmdLog {}
    variable component
    array set component {
	programs	0
	binaries	0
	headers		0
	documentation	0
	scripts		0
	demos		0
    }
    variable componentDescription
    array set componentDescription {
	programs	{Executable programs}
	binaries	{Shared and static libraries}
	headers		{Header files for program development}
	documentation	{Documentation, man pages or HTML}
	scripts		{Script libraries}
	demos		{Examples and demonstrations}
    }
    variable files
    array set files {}
    variable totalBytes 0
    variable totalFiles 0

    variable requirements {}

    variable panel 0
    variable panelList {
	Welcome Directory Components Ready Finish
    }

    variable warningMessages {}

}

# Installer::GetInstallData --
#
#	Read in configuration data
#
# Arguments:
#	srcdir	source directory
#
# Results:
#	Configuration array populated.
#	Error exits if no data found, or errors in data

proc Installer::GetInstallData srcdir {
    variable datafilename

    set filename [file join $srcdir $datafilename]
    if {![file exists $filename]} {

	set message "Installation configuration data is missing.
Unable to continue installation.

Contact vendor for complete installer"

	if {[catch {tk_messageBox -message $message -icon error -parent . -title "Missing Data" -type ok}]} {

	    puts stderr \n$message\n

	}

	exit 1
    }

    # The installer uses Tk (at the moment).
    # We'll provide a Tcl-only installer later

    if {![info exists ::tk_library]} {
	puts stderr "
This installer requires Tk.

Please restart the installer using wish,
or contact the package vendor.
"
	exit 3
    }

    # Evaluate the install data script in a slave interpreter
    # to protect the main interpreter from corruption

    set slave [interp create installData]
    foreach cmd {
	Package MajorVersion MinorVersion Patchlevel Version
	Require
	Welcome
	Component
	Executable
	File
	Files
	Index
    } {
	$slave alias $cmd Slave_$cmd $slave
    }

    set ch [open $filename]
    if {[catch {interp eval $slave [list eval [read $ch]]} msg]} {

	tk_messageBox -parent . -icon error -type ok -title "Install Error" \
		-message "An error occurred while preparing the installation configuration data.

Contact vendor for a new installer, and tell them to fix error \"$msg\""

	close $ch
	exit 2
    }
    close $ch

    return {}
}

proc Slave_Package {slave name} {
    $slave eval [list set Package $name]
    set ::Installer::Package $name
}
proc Slave_MajorVersion {slave number} {
    $slave eval [list set MajorVersion $number]
    set ::Installer::MajorVersion $number
}
proc Slave_MinorVersion {slave number} {
    $slave eval [list set MinorVersion $number]
    set ::Installer::MinorVersion $number
}
proc Slave_Patchlevel {slave name} {
    $slave eval [list set Patchlevel $name]
    set ::Installer::Patchlevel $name
}
proc Slave_Version {slave name} {
    $slave eval [list set Version $name]
    set ::Installer::Version $name
}
proc Slave_Require {slave name {number 1.0}} {
    if {[catch {package require $name $number}]} {
	tk_messageBox -parent . -title {Package Not Installed} -type ok \
		-message "Package $name version $number is required by this package,
but is not installed.

Install the required package, and then restart this installer."
	exit 4
    }

    lappend ::Installer::requirements $name number

}
proc Slave_Welcome {slave msg} {
    set ::Installer::Welcome $msg
}
proc Slave_Component {slave type} {
    set ::Installer::currentComponent $type
    set ::Installer::component($type) 1

    return {}
}
proc Slave_Executable {slave filename} {
    if {![info exists ::Installer::currentComponent]} {
	return -code error "no component has been specified"
    }
    if {[file exists $filename]} {
	lappend ::Installer::executables($::Installer::currentComponent) $filename
    } else {
	Warning "Executable file \"$filename\" not found"
    }
}
proc Slave_File {slave filename} {
    if {![info exists ::Installer::currentComponent]} {
	return -code error "no component has been specified"
    }
    if {[file exists $filename]} {
	lappend ::Installer::files($::Installer::currentComponent) $filename
    } else {
	Warning "File \"$filename\" not found"
    }
}
proc Slave_Files {slave pattern} {
    if {![info exists ::Installer::currentComponent]} {
	return -code error "no component has been specified"
    }
    set files [glob -nocomplain $pattern]
    if {[llength $files]} {
	eval lappend [list ::Installer::files($::Installer::currentComponent)] $files
    } else {
	Warning "No files matching pattern \"$pattern\" were found"
    }
}
proc Slave_Index {slave req} {
    if {![info exists ::Installer::currentComponent]} {
	return -code error "no component has been specified"
    }
    switch -- $req {
	supplied {
	    set ::Installer::pkgindex($::Installer::currentComponent) 0
	}
	needed {
	    set ::Installer::pkgindex($::Installer::currentComponent) 1
	}
	default {
	    return -code error "invalid value \"$req\", must be \"supplied\" or \"needed\""
	}
    }
}

proc Warning msg {
    append ::Installer::warningMessages $msg \n
}

proc Installer::DoInstall { package version } {
    global srcdir
    global tcl_platform
    regsub {\.} $version {} v2
    variable packagedir 
    variable component

    variable executables
    variable files

    set prefix $::TEA::prefix
    set packagedir [file join $prefix lib [string tolower $package]-$::Installer::MajorVersion.$::Installer::MinorVersion]

    # All of this stuff will move into the data file.
    # It is not used for the script-only installer

    set ::TEA::TCLXML_LIB_FILE         libtclxml2.0.so
    set ::TEA::BUILD_expat             yes
    set ::TEA::expat_TCL_LIB_FILE      libtclexpat.so
    set ::TEA::expat_LIB_DIR           ../../expat/build/linux
    set ::TEA::expat_TCL_LIB_NAME      tclexpat
    set ::TEA::expat_DIR               ../../expat/build
    set ::TEA::BUILD_xerces            no
    set ::TEA::xerces_TCL_LIB_FILE     {}
    set ::TEA::xerces_LIB_DIR          {}
    set ::TEA::xerces_DIR              {}
    set ::TEA::xerces_TCL_LIB_NAME     {}

    foreach {c b} [array get component] {
	if {$b} {
	    switch $c {
		programs {
		    foreach file $executables($c) {
			Add $srcdir -perm 0755 \
				-file $file \
				[file join $prefix bin]
		    }
		}
		binaries {
		    # TODO: Not yet implemented
		}
		headers {
		    foreach file $files($c) {
			Add $srcdir -perm 0644 \
				-file $file \
				[file join $prefix $include]
		    }
		}
		documentation {
		    foreach file $files($c) {
			switch [file extension $file] {
			    .html {
				Add $srcdir -perm 0644 \
					-file $file \
					[file join $prefix html]
			    }
			    .n {
				Add $srcdir -perm 0644 \
					-file $file \
					[file join $prefix man mann]
			    }
			}
		    }
		}
		scripts {
		    foreach file $files($c) {
			Add $srcdir -perm 0755 \
				-file $file \
				$packagedir
		    }
		}
		demos {
		    foreach file $files($c) {
			Add $srcdir -perm 0755 \
				-file $file \
				[file join $packagedir demos]
		    }
		}
	    }
	}
    }

    Install $package $version
}


#----------------------------------------------------------------------------

proc Installer::InstallDirectory { dest } {
    variable commandList
    lappend commandList [list CheckPath $dest]
}

proc Installer::Update { src dest size perm } {
    variable currentBytes
    variable totalBytes

    .install.text insert end "file copy -force $src $dest\n"
    if { [catch {file copy -force $src $dest} results] != 0 } {
	.install.text insert end "Error: $results\n" fail
    } else {
	incr currentBytes $size
    }
    global tcl_platform
    if { $tcl_platform(platform) == "unix" } {
	.install.text insert end "file attributes $dest -permissions $perm\n"
	if { [catch {file attributes $dest -permissions $perm} results] != 0 } {
	    .install.text insert end "Error: $results\n" fail
	}
    }
    set percent [expr round(double($currentBytes)/$totalBytes * 100.0)]
    .install.current configure -text "$percent% complete"
    update
}

proc Installer::InstallFile { src dest perm } {
    variable commandList
    variable totalBytes
    variable totalFiles

    if { [catch { file size $src } size ] != 0 } {
	set size 0
    }
    lappend commandList [list Update $src $dest $size $perm]
    incr totalBytes $size
    incr totalFiles
}

proc Installer::Add { dir args } {
    variable commandList 
    variable totalBytes

    if { ![file exists $dir] }  {
	error "can't find directory \"$dir\"" 
    }
    set argc [llength $args]
    set destDir [lindex $args end]
    incr argc -2
    set perm 0644

    InstallDirectory $destDir
    foreach { option value } [lrange $args 0 $argc] {
	switch -- $option {
	    "-pattern" {  
		foreach f [lsort [glob [file join $dir $value]]] { 
		    InstallFile $f [file join $destDir [file tail $f]] $perm
		}
	    }
	    "-rename" {
		set src [lindex $value 0]
		set dest [lindex $value 1]
		InstallFile [file join $dir $src] [file join $destDir $dest] $perm
	    }
    	    "-perm" {  
		set perm $value
	    }
    	    "-file" {  
		InstallFile [file join $dir $value] [file join $destDir [file tail $value]] $perm
	    }
    	    "-preprocess" {  
		set preprocessOption "-preprocess $value" 
	    }
	    default {
		error "Unknown option \"$option\""
	    }
	}
    }
}

proc Installer::CheckPath { dest } {
    set save [pwd]
    if { [file pathtype $dest] == "absolute" } {
	if { [string match  {[a-zA-Z]:*} $dest] } {
	    cd [string range $dest 0 2]
	    set dest [string range $dest 3 end]
	} else {
	    cd /
	    set dest [string range $dest 1 end]
	}
    }	
    set dirs [file split $dest]
    foreach d $dirs {
	if { ![file exists $d] } {
	    .install.text insert end "file mkdir $d\n"
	    if { [catch { file mkdir $d } result] != 0 } {
		.install.text insert end "Error: $result\n" fail
		break
	    }
	}
	if { ![file isdirectory $d] } {
	    .install.text insert end "Error: Not a directory: \"$d\"" fail
	    break
	}
	update
	cd $d 
    }
    cd $save
}


proc Installer::sed { src dest subslist perm } {

    set cmd {
	set srcId [open $src "r"]
	set destId [open $dest "w"]
 
	while { [ gets $srcId line ] >= 0 && ! [eof] } {
	    if { [regsub -all ${tag}(.+)${tag} $line \$subslist(\1) line ] } {
		subst [ set line $line ]
	    }	    
	    puts $destId $line
	}
        close $srcId
	close $destId
    }
    if { [catch $cmd result] != 0 } {
	.install.text insert end "Error: $result\n" fail
    } else {
	incr currentBytes [file size $dest]
    }

    global tcl_platform
    if { $tcl_platform(platform) == "unix" } {
	.install.text insert end "file attributes $dest -permissions $perm\n"
	if { [catch {file attributes $dest -permissions $perm} results] != 0 } {
	    .install.text insert end "Error: $results\n" fail
	}
    }
    set percent [expr round(double($currentBytes)/$totalBytes * 100.0)]
    .install.current configure -text "$percent% complete"
    update
}

proc Installer::MakePackageIndex { package version file } {

    set prefix $::TEA::prefix
    set suffix [info sharedlibextension]

    regsub {\.} $version {} version_no_dots
    set libName "${package}${version_no_dots}${suffix}"
    set libPath [file join ${prefix} bin $libName]
    set cmd [list load $libPath $package]

    if { [file exists $file] } {
	file delete $file
    }
    set cmd {
	set fid [open $file "w"] 
	puts $fid "# Package Index for $package"
	puts $fid "# generated on [clock format [clock seconds]]"
	puts $fid ""
	puts $fid [list package ifneeded $package $version $cmd]
	close $fid
    }
    if { [catch $cmd result] != 0 } {
	.install.text insert end "Error: $result\n" fail
    }
}

proc Installer::SetRegistryKey { package version valueName } {
    variable packagedir 
    global tcl_version
    
    package require registry
    set key HKEY_LOCAL_MACHINE\\Software\\$package\\$version\\$tcl_version 
    registry set $key $valueName $packagedir
}

proc Installer::Install { package version } {
    variable commandList 
    variable totalBytes 
    variable currentBytes 0
    variable totalFiles
    variable packagedir 

    .install.totals configure -text "Files: $totalFiles Size: $totalBytes"
    foreach cmd $commandList {
	if { ![winfo exists .install] } {
	    return 
	}
	if { [catch $cmd result] != 0 } {
	    .install.text insert end "Error: $result\n" fail
	} 
	update
    }
    global tcl_version tcl_platform
    set prefix $::TEA::prefix
    set name [string tolower $package]
    if {[info exists ::Installer::pkgindex(scripts)] && $::Installer::pkgindex(scripts)} {
	MakePackageIndex $package $version [file join $packagedir pkgIndex.tcl]
#	MakePackageIndex $package $version \
#		[file join $prefix lib tcl${tcl_version} ${name}${version} pkgIndex.tcl]
    }
    if { $tcl_platform(platform) == "windows" } {
	SetRegistryKey $package $version ${package}_LIBRARY
    }
    .install.cancel configure -text "Done"
}

proc Installer::Next {} {
    variable panel
    variable continue
    variable panelList

    incr panel
    set max [llength $panelList]
    if { $panel >= $max } {
	exit 0
    }
    if { ($panel + 1) == $max } {
	.next configure -text "Finish"
	.cancel configure -state disabled
    } else {
	.next configure -text "Next"
    }
    if { $panel > 0 } {
	.back configure -state normal
    }
    set continue 1
}

proc Installer::Back {} {
    variable panel
    variable continue
    
    incr panel -1 
    if { $panel <= 0 } {
	.back configure -state disabled
	set panel 0
    } else {
	.back configure -state normal
    }
    .next configure -text "Next"
    .cancel configure -state normal
    set continue 0
}

proc Installer::Cancel {} {
    exit 0
}

proc Installer::MakeLink { widget tag command } {
    $widget tag configure $tag -foreground blue -underline yes 
    $widget tag bind $tag <ButtonRelease-1> \
	"$command; $widget tag configure $tag -foreground blue"
    $widget tag bind $tag <ButtonPress-1> \
	[list $widget tag configure $tag -foreground red]
}

proc Installer::Welcome { package version }  {
    global tcl_version 
    variable Welcome
    variable requirements

    if { [winfo exists .panel] } {
	destroy .panel
    }
    text .panel -wrap word -width 10 -height 18 \
	-relief flat -padx 4 -pady 4 -cursor arrow \
	-background [. cget -bg]
    .panel tag configure welcome -font titleFont -justify center \
	-foreground navyblue
    .panel tag configure package -font hugeFont -foreground red \
	-justify center
     MakeLink .panel next "Installer::Next"
     MakeLink .panel cancel "Installer::Cancel"
    .panel insert end "Welcome!\n" welcome
    .panel insert end \n
    .panel insert end "$package Package" package
    .panel insert end \n\n
    .panel insert end "Version $version" package
    .panel insert end \n\n\n
    .panel insert end $Welcome\n

    if {[llength $requirements]} {
	.panel insert end "\nTo install this package the following packages must be installed:\n"
    }
    foreach {name number} $requirements {
	.panel insert end "$name $number\n"
    }

    .panel insert end \
	"\nPress the " "" \
	"Next" next \
	" button to continue.  Press the " "" \
	"Cancel" cancel \
	" button if you do not wish to install $package at this time."
    .panel configure -state disabled
    grid .panel -in . -column 1 -row 0 -columnspan 3 -pady 10 -padx 5 -sticky news
    tkwait variable Installer::continue
}

proc Installer::OpenDir { widget path atnode } {
    set save [pwd]
    global tcl_platform
	
    if { $tcl_platform(platform) == "windows" } {
	if { $path == "/" } {
	    foreach v [file volumes] {
		if { ![string match {[A-B]:/} $v] } {
		    .browser.h insert end $v -button yes
		}
	    }
	    return
	}
	set path [string trimleft $path /]
    } 
    cd $path/
    foreach dir [lsort [glob -nocomplain */ ]] { 
	set node [$widget insert -at $atnode end $dir]
	# Does the directory have subdirectories?
	set subdirs [glob -nocomplain $dir/*/ ]
	if { $subdirs != "" } {
	    $widget entry configure $node -button yes
	} else {
	    $widget entry configure $node -button no
	}	
    }
    cd $save
}

# Installer::CenterPanel --
#
#	Redundant because toplevel is fixed in size

proc Installer::CenterPanel { panel }  {
    update idletasks

    set x [expr ([winfo width .] - [winfo reqwidth $panel]) / 2]
    set y [expr ([winfo height .] - [winfo reqheight $panel]) / 2]
    incr x [winfo rootx .]
    incr y [winfo rooty .]
    wm geometry $panel +$x+$y
    wm deiconify $panel
}

# Installer::CenterAndSizeToplevel --
#
#	Centers the toplevel window on the screen and
#	fixes its size
#
# Arguments:
#	t	toplevel window
#
# Results:
#	Window geometry set

proc Installer::CenterAndSizeToplevel { t } {
    set width [expr [winfo screenwidth $t] / 2]
    set height [expr [winfo screenheight $t]/ 2]
    set x [expr [winfo screenwidth $t] / 4]
    set y [expr [winfo screenheight $t] / 4]
    wm geometry $t ${width}x${height}+$x+$y
    wm deiconify $t

    return {}
}

proc Installer::Browse { }  {   
    global tcl_platform 

    if { $tcl_platform(platform) == "windows" } {
	set root "C:/Program Files"
    } elseif { $tcl_platform(platform) == "macintosh" } {
	set root "/"
    } else {
	set root /usr/local
    } 
    if { [file exists $::TEA::prefix] } {
	set root $::TEA::prefix
    }

    set ::TEA::prefix  [ 
      tk_chooseDirectory \
        -title "Installer: Select Install Directory" \
        -initialdir $root ]

}

proc Installer::Browse1 { }  {
    if { [winfo exists .browser] } {
	raise .browser
	wm deiconify .browser
	return
    }
    toplevel .browser
    entry .browser.entry -bg white -borderwidth 2 -relief sunken \
	-textvariable Installer::selection
    bind .browser.entry <KeyPress-Return> {
	.browser.ok flash
	.browser.ok invoke
    }
#    blt::hierbox .browser.h -hideroot yes -separator / -height 1i -width 3i \
#	-borderwidth 2 -relief sunken -autocreate yes -trim / \
#	-yscrollcommand { .browser.sbar set } \
#	-selectcommand { 
#	    set index [.browser.h curselection]
#	    set path [lindex [.browser.h get -full $index] 0]
#	    if { $tcl_platform(platform) == "windows" } {
#		set path [string trimleft $path /]
#	    } 
#	    puts $path
#	    set Installer::selection $path
#	}

    scrollbar .browser.sbar -command { .browser.h yview }
    wm protocol .browser WM_DELETE_WINDOW { .browser.cancel invoke }
    wm transient .browser .
    frame .browser.sep -height 2 -borderwidth 1 -relief sunken
    button .browser.ok -text "Continue" -command {
	set ::TEA::prefix $Installer::selection
	grab release .browser
	destroy .browser
    }
    button .browser.cancel -text "Cancel" -command {
	grab release .browser
	destroy .browser
    }
    global tcl_platform
    if { $tcl_platform(platform) == "windows" } {
	set root "C:/"
	.browser.h open "C:/"
	.browser.h open "C:/Program Files"
	if { [file exists $::TEA::prefix] } {
	   .browser.h see $::TEA::prefix/
	}
    } else {
	set root /usr/local
	#.browser.h open $root
    }
    set Installer::selection $::TEA::prefix
    wm title .browser "Installer: Select Install Directory"
    grid .browser.entry -row 0 -column 0 -sticky ew -columnspan 2 -padx 4 -pady 4 
    grid .browser.h -row 1 -column 0 -sticky ewns -columnspan 2 -padx 4 -pady 2 
    grid .browser.sbar -row 1 -column 2 -sticky ns 
    grid .browser.sep -row 2 -column 0 -sticky ew -padx 10 -cspan 3 -pady 2 
    grid .browser.ok -row 3 -column 0  -minsize .75i -padx 4 -pady 4 
    grid .browser.cancel -row 3 -column 1  -minsize .75i -padx 4 -pady 4
#    grid configure .browser c2 r0 r2 r3 -resize none

    wm withdraw .browser
    after idle { 
	Installer::CenterPanel .browser 
	grab set .browser
    } 
}

proc Installer::Directory { package version }  {
    global tcl_version 
    if { [winfo exists .panel] } {
	destroy .panel
    }
    frame .panel
    text .panel.text -wrap word -width 10 \
	-height 10 -borderwidth 0 -padx 4 -pady 4 -cursor arrow \
	-background [.panel cget -background]
    .panel.text tag configure title -font titleFont -justify center \
	-foreground navyblue
    .panel.text insert end "Select Destination Directory\n" title
    .panel.text insert end "\n\n"
    .panel.text insert end "Please select the directory where Tcl/Tk \
$tcl_version is installed.  The package $package $version will be \
installed in the library subdirectory.\n" 
    .panel.text configure -state disabled
    frame .panel.frame -relief groove -borderwidth 2
    label .panel.frame.label -textvariable ::TEA::prefix
    button .panel.frame.button -text "Browse..." -command Installer::Browse
    grid .panel.frame.label -row 0 -column 0 -padx 4 -pady 4 -sticky w 
    grid .panel.frame.button -row 0 -column 1 -padx 4 -pady 4 -sticky e 
    grid .panel.text -row 0 -column 0 -padx 4 -pady 4 -sticky ewns 
    grid .panel.frame -row 1 -column 0 -padx 4 -sticky ew
    grid .panel -row 0 -column 1 -columnspan 3 -pady 10 -padx 5 -sticky ewns
    tkwait variable Installer::continue
}

proc Installer::Components { package version }  {

    global tcl_version 
    if { [winfo exists .panel] } {
	destroy .panel
    }
    regsub {\.} $version {} v2
    frame .panel
    text .panel.text -wrap word -width 10 \
	-height 8 -borderwidth 0 -padx 4 -pady 4 -cursor arrow \
	-background [.panel cget -background]
    .panel.text tag configure title -font titleFont -justify center \
	-foreground navyblue
    .panel.text insert end "Select Components\n" title
    .panel.text insert end "\n\n"
    .panel.text insert end "Please select the components you wish to install.\n" 
    # TODO: Minimal, Typical, Full and Customised installations
    .panel.text configure -state disabled
    frame .panel.frame -relief groove -borderwidth 2

    variable component
    variable componentDescription
    global tcl_platform
    if { $tcl_platform(platform) == "unix" } {
	set ext [info sharedlibextension]
	set sharedlib lib${package}${v2}${ext}
	set lib lib${package}${v2}.a
	set exe ""
    } elseif { $tcl_platform(platform) == "macintosh" }  {
        set ext [info sharedlibextension]
	set sharedlib lib${package}${v2}${ext}
	set lib lib${package}${v2}.shlib
	set exe ""
    } else {
	set sharedlib ${package}${v2}.dll
	set lib ${package}${v2}.lib
	set exe ".exe"
    }
    foreach {c b} [array get component] {
	if {!$b} continue
	checkbutton .panel.frame.$c \
		-text $componentDescription($c) \
		-variable Installer::component($c)
	grid .panel.frame.$c -column 0 -padx 4 -pady 4 -sticky w 
    }

    grid .panel.text  -row 0 -column 0 -padx 4 -pady 4 -sticky ewns 
    grid .panel.frame -row 1 -column 0 -padx 4 -sticky ewns

    grid .panel -row 0 -column 1 -columnspan 3 -pady 10 -padx 5 -sticky ewns
    tkwait variable Installer::continue
}

proc Installer::Ready { package version }  {
    global tcl_version 
    if { [winfo exists .panel] } {
	destroy .panel
    }
    text .panel -wrap word -width 10 -height 18 \
	-relief flat -padx 4 -pady 4 -cursor arrow \
	-background [. cget -bg]
    .panel tag configure welcome -font titleFont -justify center \
	-foreground navyblue
    .panel tag configure package -font hugeFont -foreground red \
	-justify center
     MakeLink .panel next "Installer::Next"
     MakeLink .panel cancel "Installer::Cancel"
     MakeLink .panel back "Installer::Back"
    .panel insert end "Ready To Install!\n" welcome
    .panel insert end "\n"
    .panel insert end "We're now ready to install ${package} ${version} \
and its components.\n\n"
    .panel insert end \
	"Press the " "" \
	"Next" next \
	" button to install all selected components.\n\n" "" 
    .panel insert end \
	"To reselect components, click on the " "" \
	"Back" back \
	" button.\n\n"
    .panel insert end \
	"Press the " "" \
	"Cancel" cancel \
	" button if you do not wish to install $package at this time."
    .panel configure -state disabled
    grid .panel -row 0 -column 1 -columnspan 3 -pady 10 -padx 5 -sticky ewn
    tkwait variable Installer::continue
    if { $Installer::continue } {
	Results
	update
	DoInstall $package $version
    }
}

proc Installer::Results { }  {
    if { [winfo exists .install] } {
	destroy .install
    }
    toplevel .install
    text .install.text -height 10 -width 50 -wrap none -bg white \
	-yscrollcommand { .install.ybar set } \
	-xscrollcommand { .install.xbar set } 
    .install.text tag configure fail -foreground red
    label .install.totals -text "Files: 0  Bytes: 0" -width 50
    label .install.current -text "Installing:\n" -height 2 -width 50
    scrollbar .install.ybar -command { .install.text yview }
    scrollbar .install.xbar -command { .install.text xview } -orient horizontal
    wm protocol .install WM_DELETE_WINDOW { .install.cancel invoke }
    wm transient .install .
    button .install.cancel -text "Cancel" -command {
	grab release .install
	destroy .install
    }
    grid .install.totals  -row 0 -column 0 -sticky w -columnspan 2  
    grid .install.current -row 1 -column 0 -sticky w -columnspan 2 
    grid .install.text    -row 2 -column 0 -sticky ewns
    grid .install.ybar    -row 2 -column 1 -sticky ns
    grid .install.xbar    -row 3 -column 0 -sticky ew 
    grid .install.cancel  -row 4 -column 0 -padx 4 -pady 4 -columnspan 2
 #   blt::table configure .install c1 r0 r1 r3  -resize none

    wm withdraw .install
    after idle { 
	Installer::CenterPanel .install 
	grab set .install
    } 
}

proc Installer::Finish { package version }  {
    global tcl_version 
    if { [winfo exists .panel] } {
	destroy .panel
    }
    text .panel -wrap word -width 10 -height 18 \
	-relief flat -padx 4 -pady 4 -cursor arrow \
	-background [. cget -bg]
    .panel tag configure welcome -font titleFont -justify center \
	-foreground navyblue
    .panel tag configure package -font hugeFont -foreground red \
	-justify center
    .panel insert end "Installation Completed\n" welcome
    .panel insert end "\n"
    .panel insert end "${package} ${version} is now installed.\n\n"
     MakeLink .panel finish "Installer::Next"
    .panel insert end \
	"Press the " "" \
	"Finish" finish \
	" button to exit this installation"
    .panel configure -state disabled
    grid .panel	-row 0 -column 1 -columnspan 3 -pady 10 -padx 5 -sticky ewn
    tkwait variable Installer::continue
}


set srcdir [pwd]

if { [llength $argv] > 0 } { set $TEA::prefix  [lindex $argv 0] }
if { [llength $argv] > 1 } { set srcdir        [lindex $argv 1]}
if { [llength $argv] > 2 } { set $TEA::VERSION [lindex $argv 2]}

if { ! [file exists $srcdir] } {
    error "Can't find source directory \"$srcdir\" "
}

# Read in the installation configuration data.
# This will exit if no data can be found

Installer::GetInstallData $srcdir

namespace eval TEA {

    variable prefix [file dirname [file dirname [info library]]]
    switch $::tcl_platform(platform) {
	unix {
	    # info library gives us the correct directory
	}
	macintosh {
	    set prefix $::PREFS_FOLDER
	}
	windows {
	    set prefix "C:/Program Files/Tcl"
	}
    }

    variable    exec_prefix             $prefix
    variable	PACKAGE                 $::Installer::Package
    variable	VERSION                 $::Installer::Version
    variable	RELPATH                 ..
    variable	program_transform_name  "s,x,x,"
    variable	bindir                  [file join $exec_prefix bin]
    variable	sbindir                 [file join $exec_prefix sbin]
    variable	libexecdir              [file join $exec_prefix libexec]
    variable	datadir                 [file join $prefix share]
    variable	sysconfdir              [file join $prefix etc]
    variable	sharedstatedir          [file join $prefix com]
    variable	localstatedir           [file join $prefix var]
    variable	libdir                  [file join $exec_prefix lib]
    variable	includedir              [file join $prefix include]
    variable	oldincludedir           /usr/include
    variable	infodir                 [file join $prefix info]
    variable	htmldir                 [file join $prefix html]
    variable	mandir                  [file join $prefix man]
    variable	EXEEXT                  {}

    # These hard-coded values are not used
#    variable	TCL_BIN_DIR             /usr/local/tcl/lib
#    variable	TCL_SRC_DIR             /usr/local/src/tcl/tcl8.3.2
#    variable	TCL_LIB_FILE            libtcl8.3.so
#    variable	TCL_TOP_DIR_NATIVE      /usr/local/src/tcl/tcl8.3.2
#    variable	TCL_GENERIC_DIR_NATIVE  $(TCL_TOP_DIR_NATIVE)/generic
#    variable	TCL_UNIX_DIR_NATIVE     $(TCL_TOP_DIR_NATIVE)/unix
#    variable	TCL_WIN_DIR_NATIVE      $(TCL_TOP_DIR_NATIVE)/win
#    variable	TCL_BMAP_DIR_NATIVE     $(TCL_TOP_DIR_NATIVE)/bitmaps
#    variable	TCL_TOOL_DIR_NATIVE     $(TCL_TOP_DIR_NATIVE)/tools
#    variable	TCL_PLATFORM_DIR_NATIVE $(TCL_TOP_DIR_NATIVE)/unix
    variable	TCL_BIN_DIR_NATIVE      {}
    variable	INCLUDE_DIR_NATIVE      [file join $prefix include]
    variable	CLEANFILES              {}
    variable	SHELL                   /bin/sh
}


if { $tcl_platform(platform) == "unix" } {
    font create textFont -family Helvetica -weight normal -size 11
    font create titleFont -family Helvetica -weight normal -size 14
} else {
    font create titleFont -family Arial -weight bold -size 12
    font create textFont -family Arial -weight normal -size 9
}
font create hugeFont -family {Times New Roman} -size 18 -slant italic \
    -weight bold

option add *Hierbox.openCommand	{Installer::OpenDir %W "%P" %n}

set version $::TEA::VERSION
regsub {\.} $version {} v2
set ext [info sharedlibextension]
set package $::TEA::PACKAGE

if { $tcl_platform(platform) == "unix" } {
    set sharedlib lib${package}${v2}${ext}
} elseif { $tcl_platform(platform) == "macintosh" } {
    set sharedlib ${package}${v2}${ext}
} else { #windows
    set sharedlib ${package}${v2}${ext}
}

if { [llength $argv] > 0 } { set ::TEA::prefix  [lindex $argv 0] }
image create photo openFolder -format gif -data {
R0lGODlhEAANAPIAAAAAAH9/f7+/v///////AAAAAAAAAAAAACH+JEZpbGUgd3JpdHRlbiBi
eSBBZG9iZSBQaG90b3Nob3CoIDUuMAAsAAAAABAADQAAAzk4Gsz6cIQ44xqCZCGbk4MmclAA
gNs4ml7rEaxVAkKc3gTAnBO+sbyQT6M7gVQpk9HlAhgHzqhUmgAAOw==
}
image create photo closeFolder -format gif -data {
R0lGODlhEAANAPIAAAAAAH9/f7+/v///AP///wAAAAAAAAAAACH+JEZpbGUgd3JpdHRlbiBi
eSBBZG9iZSBQaG90b3Nob3CoIDUuMAAsAAAAABAADQAAAzNIGsz6kAQxqAjxzcpvc1KWBUDY
nRQZWmilYi37EmztlrAt43R8mzrO60P8lAiApHK5TAAAOw==
}
#image create photo blt -file ${srcdir}/demos/images/blt98.gif

option add *Text.font textFont
option add *HighlightThickness 0
option add *Hierbox.icons "closeFolder openFolder"
option add *Hierbox.button yes

set color \#accaff
option add *Frame.background $color
option add *Toplevel.background $color
#option add *Button.background $color
option add *Checkbutton.background $color
option add *Label.background $color
option add *Text.background $color
. configure -bg $color
wm title . "$package $version Installer"
label .image -text [string toupper $::Installer::Package] -borderwidth 2 -relief groove
button .back -text "Back" -state disabled -command Installer::Back -underline 0
button .next -text "Next" -command Installer::Next -underline 0
button .cancel -text "Cancel" -command Installer::Cancel -underline 0
frame .sep -height 2 -borderwidth 1 -relief sunken
grid .image  -row 0 -column 0 -sticky ewns -padx 10 -pady 10 
grid .sep    -row 1 -column 0 -sticky ew -padx 4 -pady 4 -columnspan 4 -padx 5
grid .back   -row 2 -column 1 -sticky e -padx 4 -pady 4 
grid .next   -row 2 -column 2 -sticky w -padx 4 -pady 4 
grid .cancel -row 2 -column 3 -padx 20 -pady 4

grid rowconfigure . 0 -weight 1
grid rowconfigure . 2 -minsize .75i -weight 0
grid rowconfigure . 3 -minsize 0.125i

grid columnconfigure . 1 -weight 1
grid columnconfigure . 2 -weight 1
grid columnconfigure . 3 -weight 1

Installer::CenterAndSizeToplevel .

while { 1 } {
    namespace eval Installer {
	variable panel
	set cmd [lindex $panelList $panel]
	eval [list $cmd $package $version]
    }	
    update
} 
